
from .pki_builder import PKIBuilder


class IyzipyResource:
    def __init__(self, iyzipy):
        from .iyzipy import Iyzipy

        self.iyzipy: Iyzipy = iyzipy

    def request(self, method, url, request=None, pki=None):
        return self.iyzipy.client.request(method, url, request, pki=pki)

    @staticmethod
    def buyer_pki(buyer):
        pki_builder = PKIBuilder('')
        pki_builder.append('id', buyer.get('id'))
        pki_builder.append('name', buyer.get('name'))
        pki_builder.append('surname', buyer.get('surname'))
        pki_builder.append('identityNumber', buyer.get('identityNumber'))
        pki_builder.append('email', buyer.get('email'))
        pki_builder.append('gsmNumber', buyer.get('gsmNumber'))
        pki_builder.append('registrationDate', buyer.get('registrationDate'))
        pki_builder.append('lastLoginDate', buyer.get('lastLoginDate'))
        pki_builder.append('registrationAddress', buyer.get('registrationAddress'))
        pki_builder.append('city', buyer.get('city'))
        pki_builder.append('country', buyer.get('country'))
        pki_builder.append('zipCode', buyer.get('zipCode'))
        pki_builder.append('ip', buyer.get('ip'))
        return pki_builder.get_request_string()

    @staticmethod
    def address_pki(address):
        pki_builder = PKIBuilder('')
        pki_builder.append('address', address.get('address'))
        pki_builder.append('zipCode', address.get('zipCode'))
        pki_builder.append('contactName', address.get('contactName'))
        pki_builder.append('city', address.get('city'))
        pki_builder.append('country', address.get('country'))
        return pki_builder.get_request_string()

    @staticmethod
    def basket_pki(basket_items):
        basket_items_pki = []
        for item in basket_items:
            pki_builder = PKIBuilder('')
            pki_builder.append('id', item.get('id'))
            pki_builder.append_price('price', item.get('price'))
            pki_builder.append('name', item.get('name'))
            pki_builder.append('category1', item.get('category1'))
            pki_builder.append('category2', item.get('category2'))
            pki_builder.append('itemType', item.get('itemType'))
            pki_builder.append('subMerchantKey', item.get('subMerchantKey'))
            pki_builder.append_price('subMerchantPrice', item.get('subMerchantPrice'))
            basket_items_pki.append(pki_builder.get_request_string())
        return basket_items_pki

    @staticmethod
    def payment_card_pki(payment_card):
        pki_builder = PKIBuilder('')
        pki_builder.append('cardHolderName', payment_card.get('cardHolderName'))
        pki_builder.append('cardNumber', payment_card.get('cardNumber'))
        pki_builder.append('expireYear', payment_card.get('expireYear'))
        pki_builder.append('expireMonth', payment_card.get('expireMonth'))
        pki_builder.append('cvc', payment_card.get('cvc'))
        pki_builder.append('registerCard', payment_card.get('registerCard'))
        pki_builder.append('cardAlias', payment_card.get('cardAlias'))
        pki_builder.append('cardToken', payment_card.get('cardToken'))
        pki_builder.append('cardUserKey', payment_card.get('cardUserKey'))
        return pki_builder.get_request_string()

    @classmethod
    def installment_details_pki(cls, installment_details):
        installments_pki = []
        for item in installment_details:
            pki_builder = PKIBuilder('')
            pki_builder.append('bankId', item.get('bankId'))
            pki_builder.append_array('installmentPrices',
                                     cls.installment_prices_pki(item.get('installmentPrices')))
            installments_pki.append(pki_builder.get_request_string())
        return installments_pki

    @staticmethod
    def installment_prices_pki(installment_prices):
        installments_pki = []
        for item in installment_prices:
            pki_builder = PKIBuilder('')
            pki_builder.append('installmentNumber', item.get('installmentNumber'))
            pki_builder.append_price('totalPrice', item.get('totalPrice'))
            installments_pki.append(pki_builder.get_request_string())
        return installments_pki

    @staticmethod
    def card_pki(card):
        pki_builder = PKIBuilder('')
        pki_builder.append('cardAlias', card.get('cardAlias'))
        pki_builder.append('cardNumber', card.get('cardNumber'))
        pki_builder.append('expireYear', card.get('expireYear'))
        pki_builder.append('expireMonth', card.get('expireMonth'))
        pki_builder.append('cardHolderName', card.get('cardHolderName'))
        return pki_builder.get_request_string()


class SubscriptionProduct(IyzipyResource):
    def create(self, request):
        return self.request('POST', '/v2/subscription/products', request)

    def retrieve(self, reference_code):
        return self.request('GET', f'/v2/subscription/products/{reference_code}')

    def list(self):
        return self.request('GET', '/v2/subscription/products/')


class SubscriptionCustomer(IyzipyResource):
    def create(self, request):
        return self.request('POST', '/v2/subscription/customers', request)


class Subscription(IyzipyResource):
    def checkout_form(self, request):
        return self.request('POST', '/v2/subscription/checkoutform/initialize', request)


class CheckoutForm(IyzipyResource):
    def initialize(self, request):
        pki = self.to_pki_string(request)
        return self.request('POST', '/payment/iyzipos/checkoutform/initialize/auth/ecom', request, pki)

    def to_pki_string(self, request):
        pki_builder = PKIBuilder(self.iyzipy.client.resource_pki(request))
        pki_builder.append_price('price', request.get('price'))
        pki_builder.append('basketId', request.get('basketId'))
        pki_builder.append('paymentGroup', request.get('paymentGroup'))
        pki_builder.append('buyer', self.buyer_pki(request.get('buyer')))
        pki_builder.append('shippingAddress', self.address_pki(request.get('shippingAddress')))
        pki_builder.append('billingAddress', self.address_pki(request.get('billingAddress')))
        pki_builder.append_array('basketItems', self.basket_pki(request.get('basketItems')))
        pki_builder.append('callbackUrl', request.get('callbackUrl'))
        pki_builder.append('paymentSource', request.get('paymentSource'))
        pki_builder.append('currency', request.get('currency'))
        pki_builder.append('posOrderId', request.get('posOrderId'))
        pki_builder.append_price('paidPrice', request.get('paidPrice'))
        pki_builder.append('forceThreeDS', request.get('forceThreeDS'))
        pki_builder.append('cardUserKey', request.get('cardUserKey'))
        pki_builder.append_array('enabledInstallments', request.get('enabledInstallments'))
        pki_builder.append('debitCardAllowed', request.get('debitCardAllowed'))

        return pki_builder.get_request_string()



import base64
import hashlib
import hmac
import json
import random
import re
import string
from urllib.parse import urljoin

from requests import Session, Response as BaseResponse

from .resources import SubscriptionProduct, SubscriptionCustomer, Subscription, CheckoutForm
from .pki_builder import PKIBuilder


class Iyzipy:
    def __init__(self, options: dict):
        self.base_url: str = options["base_url"]
        self.api_key: str = options["api_key"]
        self.secret_key: str = options["secret_key"]

        self.client = Client(self)

        self.subscription_product = SubscriptionProduct(self)
        self.subscription_customer = SubscriptionCustomer(self)
        self.subscription = Subscription(self)
        self.checkout_form = CheckoutForm(self)


class Response(dict):
    def __init__(self, res: BaseResponse):
        data = res.json()
        super().__init__(data)

        self.response = res

    @property
    def status_code(self):
        return self.response.status_code


class Client:
    RANDOM_STRING_SIZE = 8
    RE_SEARCH_V2 = r'/v2/'
    header = {
        "Accept": "application/json",
        "Content-type": "application/json",
        'x-iyzi-client-version': 'iyzipay-python-1.0.32'
    }

    def __init__(self, iyzipy: 'Iyzipy'):
        self.session = Session()
        self.iyzipy = iyzipy

    def get_url(self, url):
        return urljoin(self.iyzipy.base_url, url)

    def request(self, method, url, request_body_dict=None, pki=None):
        body_str = json.dumps(request_body_dict)
        header = self.get_http_header(url, body_str, pki)

        url = self.get_url(url)

        response = self.session.request(method, url, data=body_str, headers=header)

        return Response(response)

    def get_http_header(self, url, body_str=None, pki_string=None):
        random_str = self.generate_random_string(self.RANDOM_STRING_SIZE)
        self.header.update({'x-iyzi-rnd': random_str})
        if re.search(self.RE_SEARCH_V2, url, re.IGNORECASE) is not None:
            return self.get_http_header_v2(url, random_str, body_str)
        else:
            return self.get_http_header_v1(pki_string, random_str)

    def get_http_header_v1(self, pki_string, random_str=None):
        if pki_string is not None:
            self.header.update(
                {'Authorization': self.prepare_auth_string(random_str, pki_string)})
        return self.header

    def get_http_header_v2(self, url, random_str, body_str):
        url = url.split('?')[0]
        hashed_v2_str = self.generate_v2_hash(url, random_str, body_str)
        self.header.update(
            {'Authorization': 'IYZWSv2 %s' % hashed_v2_str})
        return self.header

    def generate_v2_hash(self, url, random_str, body_str):
        secret_key = bytes(self.iyzipy.secret_key.encode('utf-8'))
        msg = (random_str + url + body_str).encode('utf-8')

        hmac_obj = hmac.new(secret_key, digestmod=hashlib.sha256)
        hmac_obj.update(msg)
        signature = hmac_obj.hexdigest()
        authorization_params = [
            'apiKey:' + self.iyzipy.api_key,
            'randomKey:' + random_str,
            'signature:' + signature
        ]
        return base64.b64encode('&'.join(authorization_params).encode()).decode()

    def get_plain_http_header(self):
        return self.get_http_header_v1(None)

    def prepare_auth_string(self, random_str, pki_string):
        hashed = self.generate_hash(self.iyzipy.api_key, self.iyzipy.secret_key, random_str, pki_string)
        return self.format_header_string(self.iyzipy.api_key, hashed)

    @staticmethod
    def generate_random_string(size):
        return "".join(
            random.SystemRandom().choice(string.ascii_letters + string.digits) for _ in
            range(size))

    @staticmethod
    def generate_hash(api_key, secret_key, random_string, pki_string):
        hash_str = api_key + random_string + secret_key + pki_string
        hex_dig = hashlib.sha1(hash_str.encode()).digest()

        return base64.b64encode(hex_dig)

    @staticmethod
    def format_header_string(api_key, hashed):
        hashed = hashed.decode('utf-8')
        return 'IYZWS %s:%s' % (api_key, hashed)

    @staticmethod
    def resource_pki(request):
        return 'locale=' + request.get('locale', "") + \
               (',conversationId=' + request.get('conversationId') + ',' if request.get('conversationId') else ',')



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
import shutil
import sys

import re
from setuptools import setup, find_packages
from codecs import open
from os import path
import os

try:
    from pip.req import parse_requirements
except ImportError:
    # pip >= 10
    from pip._internal.req import parse_requirements

here = path.abspath(path.dirname(__file__))


def get_requirements(requirements_file):
    """Use pip to parse requirements file."""
    requirements = []
    if path.isfile(requirements_file):
        for req in parse_requirements(requirements_file, session="hack"):
            if req.markers:
                requirements.append("%s;%s" % (req.req, req.markers))
            else:
                requirements.append("%s" % req.req)
    return requirements


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('iyzipy')


if __name__ == "__main__":
    INSTALL_REQUIRES = get_requirements(path.join(here, "requirements", "requirement-main.txt"))
    TESTING_REQUIRES = get_requirements(path.join(here, "requirements", "requirement-main.txt"))

    # Get the long description from the README file
    with open(path.join(here, 'README.MD'), encoding='utf-8') as f:
        long_description = f.read()


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree('dist')
    # shutil.rmtree('build')
    shutil.rmtree('iyzipy.egg-info')
    sys.exit()


setup(
    name='iyzipy',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='Iyzico Python Client',
    long_description=long_description,
    long_description_content_type="text/markdown",

    # The project's main homepage.
    url='https://gitlab.com/kalaomer/iyzipy',

    # Author details
    author='kalaomer',
    author_email='kalaomer@hotmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    # What does your project relate to?
    keywords='iyzico client payment',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=INSTALL_REQUIRES,

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'test': TESTING_REQUIRES,
    }
)
